/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  chimeclk by Jack Botner

  chimeclk.c

  This program implements a chiming clock with the following hardware:
  - a KTM-S1201 12 x 1 serial LCD display
  - a CTS rotary encoder with pushbutton
  - DS1305 real time clock

  Processor: ATmega88P
      Clock: Internal, 8.0 MHz

  20210103 Implement a quiet period so the alarm does not 
           annoy anyone at night
-----------------------------------------------------------------*/
#include <stdint.h>
#include <avr/io.h>
//#include <avr/wdt.h>
//#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay_basic.h>
#include <stdlib.h>
#include <string.h>
#include "chimeclk.h"
#include "ktms1201.h"
#include "nju6355.h"

#define NR_MAIN		6			// number of main menu items

// Global Variables

RTCPARMS stRtc;						// real time clock data

volatile uint8_t ucEventFlags=0;		// program event flags
volatile uint8_t ucDebouncedState=0;	// debounced state of the switches
volatile uint16_t uiMsPressed=0;		// how long encoder button pressed
volatile uint8_t  ucPlayLctr=0;			// 1/3 second ticks to maintain PlayL
volatile uint16_t uiLcdMaintCtr=0;		// LCD maintenance - counts 1/3 seconds

static uint8_t ucMode=MODE_RUN;		// program mode
static uint8_t ucLastMinute=5;		// the last minute from RTC
static uint8_t ucAlarmOn=1;			// alarm on/off

// Internal Functions

static void format_and_display_time(void);
static uint8_t main_menu(void);
static void system_init(void);

/*-----------------------------------------------------------------
  main()
-----------------------------------------------------------------*/
int main( void )
{
  uint8_t ii;
  uint8_t ucIndex;

  system_init();          		// initialize system

  set_bit( PORTD, PORTD6 );		// power up the LCD
  for ( ii=0; ii<2; ++ii )		// wait 50 ms
	_delay_loop_2( 50000 );		// (25 ms)
  LCD_init();					// initialize the LCD
  LCD_puts2( "ChimeClk 1.1", 0 );

  rtc_init();					// initialize the rtc

  for ( ii=0; ii<80; ++ii )		// 2 s
	_delay_loop_2( 50000 );		// 25 ms

  ucEventFlags = 0;				// clear all events

  while(1)
  {
    if ( ucMode == MODE_RUN )
	{
	  // In RUN mode we display the time and watch for the
	  // encoder button, which puts us into setup mode.

	  if ( test_bit( ucEventFlags, EVENT_READ_RTC ) )
	  {
		rtc_read();
		format_and_display_time();

		if ( ucAlarmOn )
		{
          // 20210103 Implement a quiet period so the alarm does
          // not annoy anyone at night
          if ( stRtc.rtc_hours > 7 && stRtc.rtc_hours < 23 )
          {
		    if ( !stRtc.rtc_mins && ucLastMinute == 59 )
		    {
		      // Time just changed to the hour so sound the 
		      // long alarm
		      clear_bit( PORTC, PORTC4 );		// PLAYE on
		      for ( ii=0; ii<4; ++ii )			// 100 ms
		        _delay_loop_2( 50000 );			// 25 ms
	  	      set_bit( PORTC, PORTC4 );			// PLAYE off
		    }

		    if ( stRtc.rtc_mins == 30 && ucLastMinute == 29 )
		    {
		      // Time just changed to the half-hour so sound
		      // the short alarm
	  	      clear_bit( PORTC, PORTC5 );		// PLAYL on
		      ucPlayLctr = 9;
		    }
          }
		}

		ucLastMinute = stRtc.rtc_mins;
		clear_bit( ucEventFlags, EVENT_READ_RTC );
	  }

 	  if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
	  {
	    ucMode = MODE_PROGRAM;
	    clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
	    clear_bit( ucEventFlags, EVENT_ENCODER_UP );	// just in case
	    clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	  }

	  if ( test_bit( ucEventFlags, EVENT_LCD_MAINT ) )
	  {
	    // I do this because the LCD seems to conk out randomly after
		// a few hours/days/weeks.
		clear_bit( PORTD, PORTD6 );		// power down the LCD
		_delay_loop_2( 50000 );			// 25 ms
		set_bit( PORTD, PORTD6 );		// power up the LCD
		_delay_loop_2( 50000 );			// 25 ms
		LCD_init();						// initialize the LCD

		uiLcdMaintCtr = 0;
		clear_bit( ucEventFlags, EVENT_LCD_MAINT );
	  }
	}
	else	// MODE_PROGRAM
	{
	  // In PROGRAM mode we display the menu and perform user requests.

	  ucIndex = main_menu();	// returns index of menu item to process
	  switch ( ucIndex )
	  {
	    case INDEX_RUN:
		  // Go back to RUN mode
	      ucMode = MODE_RUN;
		  break;

	    case INDEX_SETCLK:
		  // Set the RTC date and time.
		  program_clock();
	      ucMode = MODE_RUN;
		  break;

	    case INDEX_ALARM:
		  ucAlarmOn = ( ucAlarmOn ) ? 0 : 1; 
	      ucMode = MODE_RUN;
		  break;

	    case INDEX_ALARMTEST1:
	  	  clear_bit( PORTC, PORTC5 );		// PLAYL on
		  ucPlayLctr = 9;
		  break;

	    case INDEX_ALARMTEST2:
		  clear_bit( PORTC, PORTC4 );		// PLAYE on
		  for ( ii=0; ii<4; ++ii )			// 100 ms
		    _delay_loop_2( 50000 );			// 25 ms
	  	  set_bit( PORTC, PORTC4 );			// PLAYE off
		  break;

	    case INDEX_CLKTEST:
		  // The clock test outputs a signal on the data pin that
		  // can be measured to check the accuracy of the clock.
	      rtc_test();		// this function does not return.
	  }
	}
  }
}

/*-----------------------------------------------------------------
  main_menu()

  Displays the main menu.

  Returns: index of selected menu item.
-----------------------------------------------------------------*/
static uint8_t main_menu()
{
  static volatile uint8_t ucIndex=0;

  static char *msgs[NR_MAIN] = 
  {
    "Run", "Set Clock", NULL, "Chime Test 1", "Chime Test 2", "CLK TEST"
  };

  while ( 1 )
  {
	if ( ucIndex == INDEX_ALARM )
	  msgs[INDEX_ALARM] = ( ucAlarmOn ) ? "Chime Off" : "Chime On";

	LCD_puts2( msgs[ucIndex], 0 );

 	if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
	{
	  clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
	  return ucIndex;
	}

 	if ( test_bit( ucEventFlags, EVENT_ENCODER_UP ) )
	{
	  ++ucIndex;
	  if ( ucIndex == NR_MAIN )
	    ucIndex = 0;
	  clear_bit( ucEventFlags, EVENT_ENCODER_UP );
	  continue;
	}

 	if ( test_bit( ucEventFlags, EVENT_ENCODER_DOWN ) )
	{
	  if ( ucIndex )
		--ucIndex;
	  else
	    ucIndex = NR_MAIN - 1;
	  clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	}
  }
}

/*-----------------------------------------------------------------
  format_and_display_time()

  Display the time on the LCD: HH MM SS
  The clock runs in 24 hour format but we want to display it
  in 12 hour format.
-----------------------------------------------------------------*/
static void format_and_display_time()
{
  char	buffer[12], temp[4];
  uint8_t ucHr;

  ucHr = stRtc.rtc_hours; 
  if ( ucHr > 12 ) ucHr -= 12; 
  utoa( (uint16_t) ucHr, temp, 10 );
  if ( strlen( temp ) == 1 )
  {
	buffer[0] = ' ';
	buffer[1] = temp[0];
  }
  else
  {
	buffer[0] = temp[0];
	buffer[1] = temp[1];
  }
  buffer[2] = ' ';

  utoa( (uint16_t) stRtc.rtc_mins, temp, 10 );
  if ( strlen( temp ) == 1 )
  {
	buffer[3] = '0';
	buffer[4] = temp[0];
  }
  else
  {
	buffer[3] = temp[0];
	buffer[4] = temp[1];
  }
  buffer[5] = ' ';

  utoa( (uint16_t) stRtc.rtc_secs, temp, 10 );
  if ( strlen( temp ) == 1 )
  {
	buffer[6] = '0';
	buffer[7] = temp[0];
  }
  else
  {
	buffer[6] = temp[0];
	buffer[7] = temp[1];
  }

  buffer[8] = '\0';
  LCD_puts( buffer, 1 );

  if ( ucAlarmOn )
	LCD_insert_decimal( 10 );
}

/*-----------------------------------------------------------------
  system_init()

  PORTB: PB0 (input) DS1305 interrupt0 
         PB1 (input) not used 
         PB2 (input) Rotary Encoder switch
         PB3 (input) ISP MOSI
         PB4 (input) ISP MISO
         PB5 (input) ISP SCK
         PB6 (input) Rotary Encoder "B"
         PB7 (input) Rotary Encoder "A"

  PORTC: PC0 (output) RTC CE
         PC1 (output) RTC SCLK
         PC2 (output) RTC Data
         PC3 (output) not used
         PC4 (output) ISD1416 !PLAYE
         PC5 (output) ISD1416 !PLAYL
         PC6 (output) ISP !Reset 

  PORTD: PD0 (output) LCD !SCK
         PD1 (output) LCD SI
         PD2 (output) LCD C/!D
         PD3 (output) LCD !Reset
         PD4 (output) LCD !Busy
         PD5 (output) LCD !CS
         PD6 (output) LCD Vcc
         PD7 (output) diagnostic LED
-----------------------------------------------------------------*/
static void system_init()
{
  // The fuses are set to provide an internal clock, running at
  // 8.0 MHz with a prescaler of 8. Change the prescaler from
  // 8 to 1. The procedure requires two steps as per the device
  // specifications.
  CLKPR = 0b10000000;       // set clock prescaler change enable
  CLKPR = 0b00000000;       // set prescaler to 1

  // Set up the I/O ports directions, pull-ups
  DDRB  = 0b00000000;		// all ports input
  PORTB = 0b11000100;       // initialize PortB; pull-ups on pb2, 6, 7

  DDRC  = 0b11111111;		// all ports output
  PORTC = 0b00110000;       // initialize PortC

  DDRD  = 0b11101111;		// all ports output except PD4
  PORTD = 0b00101001;       // initialize PortD

  // Initialize Timer1.
  // Timer1 is used to create 1/3 s intervals for scheduling events
  TCCR1A = 0b00000000;		// normal operation, no OC1A, no OC1B
  TCCR1B = 0b00000100;      // internal clock, prescaler = 256
  TCNT1 = INIT_TIMER1;
  set_bit( TIMSK1, TOIE1 );	// enable timer/counter1 overflow interrupt

  // Initialize Timer2. Timer2 is used to generate interrupts
  // every 2 milliseconds for switch debouncing.
  TCCR2A = 0b00000000;		// normal port operation
  TCCR2B = 0b00000100;		// normal operation, prescaler=64
  TCNT2 = INIT_TIMER2;
  set_bit( TIMSK2, TOIE2 );	// enable timer/counter2 overflow interrupt

  sei();					// enable global interrupts
}
