/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  chimeclk by Jack Botner

  interrupt.c

  This module contains all the interrupt handlers, including
  button debouncing logic.
-----------------------------------------------------------------*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include "chimeclk.h"

// Global variables

extern volatile uint8_t  ucEventFlags;			// system event flags
extern volatile uint8_t  ucDebouncedState;		// debounced state of the switches
extern volatile uint16_t uiMsPressed;			// how long encoder button pressed
extern volatile uint8_t  ucPlayLctr;				// 1/3 second ticks to maintain PlayL
extern volatile uint16_t uiLcdMaintCtr;			// LCD maintenance - counts 1/3 seconds

static uint8_t  ucPreviousState=0; 		// previous switch state
static uint8_t  ucNrUps=0, ucNrDowns=0; // CTS Encoder logic

// Internal Functions

static void analyze_switch(void);

/*-----------------------------------------------------------------
  ISR(TIMER1_OVF_vect)

  Timer/Counter 1 overflow interrupt handler
  Used to provide 1/3 second intervals for reading and displaying
  the RTC. Also used to hold PlayL signal for one second.

  System clock frequency = 8 mHz
  System clock period = 125 ns
  Prescaler = 256
  Timer count period = 32 us
  Counts = 10417
  Interrupt period = 1/3 sec

  The idea is to refresh the display frequently enough so that
  the seconds appear to increment regularly. 
-----------------------------------------------------------------*/
ISR(TIMER1_OVF_vect)
{
  set_bit( ucEventFlags, EVENT_READ_RTC );

  if ( ucPlayLctr )
  {
    // ucPlayLctr is used to hold PlayL for one second.
    --ucPlayLctr;
	if ( !ucPlayLctr )
	  set_bit( PORTC, PORTC5 );		// PLAYL off
  }

  ++uiLcdMaintCtr;
  if ( uiLcdMaintCtr >= LCD_MAINT_COUNT )
	set_bit( ucEventFlags, EVENT_LCD_MAINT );

  TCNT1 = INIT_TIMER1;
}

/*-----------------------------------------------------------------
  ISR(TIMER2_OVF_vect)

  Timer 2 overflow interrupt handler - tests for button presses

  System clock frequency = 8 mHz
  System clock period = 125 ns
  Prescaler = 64
  Timer count period = 4 us
  Counts = 250
  Interrupt period = 2 ms
------------------------------------------------------------------*/
ISR(TIMER2_OVF_vect)
{
  static volatile uint8_t ucFirst=1;
  static volatile uint8_t ucIndex=0;             	// index into ucState
  static volatile uint8_t ucState[MAX_CHECKS] =  	// array containing bounce status
        	{ 0, 0, 0, 0, 0, 0, 0, 0 };

  uint8_t ii, ucResult, ucSwIn=0;

  // We use the technique documented in "A Guide to Debouncing" by
  // Jack Ganssle to debounce up to 8 inputs (3 in this case).

  // Read current switch states
  if ( !test_bit( ENCODER_PORT, ENCODER_A ) )
    set_bit( ucSwIn, ENC_SWITCH_A );
  if ( !test_bit( ENCODER_PORT, ENCODER_B ) )
    set_bit( ucSwIn, ENC_SWITCH_B );
  if ( !test_bit( ENCODER_PORT, ENCODER_PB ) )
    set_bit( ucSwIn, ENC_SWITCH_PB );

  ucState[ucIndex] = ucSwIn;

  ++ucIndex;
  if ( ucIndex == MAX_CHECKS )
    ucIndex = 0;

  // And all columns of the array
  for ( ii=0, ucResult=0xff; ii<(MAX_CHECKS-1); ++ii )
    ucResult = ucResult & ucState[ii];

  ucDebouncedState = ucResult;

  // The 1st time only, set the previous switch state to the
  // current switch state.
  if ( ucFirst )
  {
    ucPreviousState = ucDebouncedState;
	ucFirst = 0;
  }

  // Since we're here every millisecond, keep count
  // of how many milliseconds the encoder button is pressed

  if ( test_bit( ucDebouncedState, ENC_SWITCH_PB ) )
    ++uiMsPressed;
  else
    uiMsPressed = 0;

  if ( ucDebouncedState != ucPreviousState )
    analyze_switch();		// convert switch change to event

  TCNT2 = INIT_TIMER2;
}

/*-----------------------------------------------------------------
  analyze_switch()

  Analyzes the rotary encoder to determine if the shaft rotation
  is up or down, or if the pushbutton has been pressed.

  Note. Since I am using a CTS Series 290 Encoder, things are a
  bit more complicated. The problem is, for each rotary detent,
  the encoder outputs four states instead of the expected one.
  So in effect you get four "ups" or four "downs" for each
  detent. To compensate, count to 4.
-----------------------------------------------------------------*/
static void analyze_switch()
{
  uint8_t	ucNow, ucThen;

  if ( test_bit( ucDebouncedState, ENC_SWITCH_PB ) && 
       !test_bit( ucPreviousState, ENC_SWITCH_PB ) )
  {
    // The encoder button has been pressed
	set_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
  }

  // Convert the position of the encoder to a number 0..3
  ucNow = ucDebouncedState & 0x03;
  ucThen = ucPreviousState & 0x03;

  // Given the A output in bit 0 and B output in bit 1 positions, the
  // encoder generates the values 0, 1, 3, 2 as it is rotated clockwise.

  if ( ucNow == 0 )
  {
    if ( ucThen == 2 )
	{
	  ++ucNrUps;
	  ucNrDowns = 0;
	}
	else
	  if ( ucThen == 1 )
	  {
	    ++ucNrDowns;
		ucNrUps = 0;
	  }
  }
  else
	if ( ucNow == 1 )
	{
      if ( ucThen == 0 )
	  {
	    ++ucNrUps;
		ucNrDowns = 0;
	  }
	  else
	    if ( ucThen == 3 )
		{
		  ++ucNrDowns;
		  ucNrUps = 0;
		}
	}
	else
	  if ( ucNow == 3 )
	  {
    	if ( ucThen == 1 )
		{
	      ++ucNrUps;
		  ucNrDowns = 0;
		}
		else
	 	  if ( ucThen == 2 )
		  {
		    ++ucNrDowns;
		    ucNrUps = 0;
		  }
	  }
	  else	// ucNow == 2
	  {
    	if ( ucThen == 3 )
		{
	      ++ucNrUps;
		  ucNrDowns = 0;
		}
		else
	 	  if ( ucThen == 0 )
		  {
			++ucNrDowns;
			ucNrUps = 0;
		  }
	  }

  if ( ucNrUps == 4 )
  {
	set_bit( ucEventFlags, EVENT_ENCODER_UP );
	ucNrUps = ucNrDowns = 0;
  }

  if ( ucNrDowns == 4 )
  {
	set_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	ucNrUps = ucNrDowns = 0;
  }

  ucPreviousState = ucDebouncedState;
}
