/*-----------------------------------------------------------------
  All code is provided for EDUCATIONAL purposes ONLY. There is no
  warranty and no support.

  Released under the MIT License  - see
  http://www.opensource.org/licenses/mit-license.php

  chimeclk by Jack Botner

  program.c

  Routines to run the programming dialogs.
-----------------------------------------------------------------*/
#include <stdint.h>
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "chimeclk.h"
#include "ktms1201.h"
#include "nju6355.h"

#define UPDATE_THE_DISPLAY	1
#define WAITFOR_EVENT		2
#define ST_UPDATE			5
#define ST_DONE				9

// Global variables

extern uint8_t ucEventFlags;
extern uint16_t uiMsPressed;
extern RTCPARMS stRtc;

// Internal functions

static uint8_t update_number( const char *pcText, uint8_t, uint8_t, uint8_t );
static void strextnd( char *s1, uint8_t fillchar, uint8_t newlength );

/*-----------------------------------------------------------------
  program_clock()

  Manage programming the clock dialogs. 
-----------------------------------------------------------------*/
void program_clock()
{
  const char *msg1  = "Year:";
  const char *msg2  = "Month:";
  const char *msg3  = "Day:";
  const char *msg4  = "Hour:";
  const char *msg5  = "Minute:";
  uint8_t ucHigh, ucLow, ucInit, ucRc;

  // Program the year
  ucLow = 0;
  ucHigh = 50;
  ucInit = stRtc.rtc_year;
  ucRc = update_number( msg1, ucInit, ucLow, ucHigh );
  if( ucRc != stRtc.rtc_year )
	stRtc.rtc_year = ucRc;

  // Program the month
  ucLow = 1;
  ucHigh = 12;
  ucInit = stRtc.rtc_month;
  ucRc = update_number( msg2, ucInit, ucLow, ucHigh );
  if( ucRc != stRtc.rtc_month )
	stRtc.rtc_month = ucRc;

  // Program the day
  ucLow = 1;
  ucHigh = 31;
  ucInit = stRtc.rtc_day;
  ucRc = update_number( msg3, ucInit, ucLow, ucHigh );
  if( ucRc != stRtc.rtc_day )
	stRtc.rtc_day = ucRc;

  // Program the hour
  ucLow = 0;
  ucHigh = 23;
  ucInit = stRtc.rtc_hours;
  ucRc = update_number( msg4, ucInit, ucLow, ucHigh );
  if( ucRc != stRtc.rtc_hours )
	stRtc.rtc_hours = ucRc;

  // Program the minute
  ucLow = 0;
  ucHigh = 59;
  ucInit = stRtc.rtc_mins;
  ucRc = update_number( msg5, ucInit, ucLow, ucHigh );
  if( ucRc != stRtc.rtc_mins )
	stRtc.rtc_mins = ucRc;

  // Update the rtc, since seconds is always assumed to be zero.
  rtc_write();
}

/*-----------------------------------------------------------------
  update_number(...)

  Show a number after the prompt text. Allow the encoder to
  increase or decrease the number. Return the result.
-----------------------------------------------------------------*/
static uint8_t update_number( const char *pcText, uint8_t initial_value, 
							  uint8_t low_bound, uint8_t high_bound )
{
  uint8_t	ucResult=255, ucTemp;
  char		buffer[12];

  // Make sure initial_value is reasonable (for example it won't be
  // when the rtc is first powered on)
  if ( initial_value < low_bound || initial_value > high_bound )
    ucTemp = low_bound;
  else
	ucTemp = initial_value;

  while ( 1 )
  {
	//wdt_reset();

	if ( ucResult != ucTemp )
	{
	  // Display current value
	  strcpy( buffer, pcText );
	  strextnd( buffer, ' ', 8 );
	  utoa( (uint16_t) ucTemp, buffer+8, 10 );
	  LCD_puts2( buffer, 0 );
	  ucResult = ucTemp;
	}

  	if ( test_bit( ucEventFlags, EVENT_ENCODER_UP ) )
	{
	  clear_bit( ucEventFlags, EVENT_ENCODER_UP );
	  // increment the value
	  ++ucTemp;
	  if ( ucTemp > high_bound )
	    ucTemp = low_bound;
	  continue;
	}

  	if ( test_bit( ucEventFlags, EVENT_ENCODER_DOWN ) )
	{
	  clear_bit( ucEventFlags, EVENT_ENCODER_DOWN );
	  // decrement the value
	  if ( ucTemp == low_bound )
		ucTemp = high_bound;
	  else
		--ucTemp;
	  continue;
	}

	if ( test_bit( ucEventFlags, EVENT_BUTTON_PRESSED ) )
	{
	  clear_bit( ucEventFlags, EVENT_BUTTON_PRESSED );
	  break;
	}
  }

  return ucResult;
}

/*-----------------------------------------------------------------
  strextnd(...)

  extend a string with fill character
-----------------------------------------------------------------*/
static void strextnd( char *s1, uint8_t fillchar, uint8_t newlength )
{
  uint8_t ii, nn;

  nn = strlen( s1 );
  if ( nn >= newlength ) return;

  for ( ii=nn; ii<newlength; ++ii ) *(s1+ii) = fillchar;
  *( s1+newlength ) = '\0';
}
